@@ -8,7 +8,7 @@ static char SccsId[] = "%W%  %G%";
  * Subroutine:	string_cmdline()		returns: void
  * Subroutine:	usage()				returns: int
  * Xlib calls:	none
- * Copyright:	1989-2000 Smithsonian Astrophysical Observatory
+ * Copyright:	1989-2002 Smithsonian Astrophysical Observatory
  *		You may do anything you like with this file except remove
  *		this copyright.  The Smithsonian Astrophysical Observatory
  *		makes no representations about the suitability of this
@@ -444,6 +444,7 @@ int usage ( what, argc, argv, first, last )
   (void)fprintf(stderr," [(-bordercolor) (-bc) <color>]\n");
   (void)fprintf(stderr,"  [-lfrac <fraction>] [(-wcscom <command>]\n");
   (void)fprintf(stderr,"  [(-ccursor) (-cc)] [-f<1-9> <command>]\n");
+  (void)fprintf(stderr,"  [(-region) (-reg) <PROS region file>]\n");
   (void)fprintf(stderr,"  [-wcsout <output coordinate system>]\n");
   (void)fprintf(stderr,"  [-help] [-version]\n");
 #ifdef ALLIANT
@@ -488,4 +489,6 @@ int usage ( what, argc, argv, first, last )
  * Feb 23 1999	Doug Mink  Add -nimage and -face to command list
  *
  * Mar  7 2000	Doug Mink  Change oldwcs to wcsproj
+ *
+ * Apr  8 2002	Doug Mink  Add -reg and -region to command list
  */
