@@ -5,7 +5,7 @@ static char SccsId[] = "%W%  %G%";
 /* Module:	disppsct.c (Display Postscript)
  * Purpose:	Dump contents of the display window to a postscript printer.
  * Subroutine:	screen_dump()			returns: void
- * Copyright:	1989,1998 NOAO & Smithsonian Astrophysical Observatory
+ * Copyright:	1989-2003 NOAO & Smithsonian Astrophysical Observatory
  *		You may do anything you like with this file except remove
  *		this copyright.  The Smithsonian Astrophysical Observatory
  *		makes no representations about the suitability of this
@@ -18,6 +18,7 @@ static char SccsId[] = "%W%  %G%";
  *              {3} Change R_DISPOSE printer from lw to lp	 1 May 1995
  *              {4} Update VMS fixes (Harry Payne STScI)         1 Jun 1998
  *              {5} Fix bug in VMS ifdef (Allan Brighton ESO)   20 Jul 1998
+ *              {6} Drop -r argument in R_DISPOSE and P_DISPOSE  3 Sep 2003
  *		{n} <who> -- <does what> -- <when>
  */
 
@@ -62,8 +63,8 @@ screen_dump ()
 #define R_DISPOSE "print/delete %s"	/* printer dispose command */
 #define P_DISPOSE "print/delete/queue=%s %s"  /* dispose to user's printer */
 #else
-#define	R_DISPOSE "lpr -Plp60_1 -r %s"	/* printer dispose command	*/
-#define P_DISPOSE "lpr -P%s -r %s"	/* dispose to user's printer	*/
+#define	R_DISPOSE "lpr %s"		/* printer dispose command	*/
+#define P_DISPOSE "lpr -P%s %s"		/* dispose to user's printer	*/
 #endif
 #define	SEGSIZE		16		/* output segment size (bytes)	*/
 #define	SEGBITS		(SEGSIZE*8)
