@@ -7,7 +7,7 @@ static char SccsId[] = "%W%  %G%";
  * Subroutine:	key_response()			returns: void
  * Subroutine:	set_iraf_key_trigger()		returns: void
  * Xlib calls:	XWarpPointer(), XSync()
- * Copyright:	1999 Smithsonian Astrophysical Observatory
+ * Copyright:	2003 Smithsonian Astrophysical Observatory
  *		You may do anything you like with this file except remove
  *		this copyright.  The Smithsonian Astrophysical Observatory
  *		makes no representations about the suitability of this
@@ -84,6 +84,9 @@ static double oldypix = 0.0;
 /*  Subroutine:	key_response
  *  Purpose:	Respond to a key stroke
  */
+
+static double oldxpos, oldypos;
+
 void
 key_response()
 {
@@ -110,7 +113,7 @@ key_response()
   static void PrintFITSHead();
   void toggle_region_visibility(), toggle_region_labeling();
   double xbuf, ybuf, xfile, yfile;
-  double oldxpos, oldypos, newxpos, newypos, distance;
+  double newxpos, newypos, distance;
   char sysout0[16];
   char wcstr[32];
   int lwcstr = 32;
@@ -529,8 +532,18 @@ key_response()
 	    newxpos = wcs->xpos;
 	    newypos = wcs->ypos;
 	    distance = wcsdist (oldxpos, oldypos, newxpos, newypos);
-	    printf ("Separation is %.5f degrees or %.2f arcseconds\n",
-		    distance, distance/3600.0);
+	    if (distance < 1.0) {
+		printf ("Separation is %.3f arcseconds\n", distance*3600.0);
+		printf ("dRA = %.3f arcsec, dDec = %.3f arcsec\n",
+                   (newxpos-oldxpos)*3600.0 * cosdeg (0.5 * (oldypos+newypos)),
+                   (newypos - oldypos) * 3600.0);
+		}
+	    else {
+		printf ("Separation is %.5f degrees\n", distance);
+		printf ("dRA = %.5f degrees, dDec = %.5f degrees\n",
+                   (newxpos-oldxpos) * cosdeg (0.5 * (oldypos+newypos)),
+                   (newypos - oldypos));
+		}
 	    oldxpos = wcs->xpos;
 	    oldypos = wcs->ypos;
 	    }
@@ -652,7 +665,7 @@ key_response()
 	printf ("H toggles cursor coordinates between degrees and hh: dd: \n");
 	printf ("I tracks cursor coordinates in coordinate system of image\n");
 	printf ("J tracks cursor coordinates in J2000 (FK5) coordinates\n");
-	printf ("L prints distance since last position, assuming same system\n");
+	printf ("L prints table of pixels values around mouse without scaling\n");
 	printf ("N enters New command-line arguments\n");
 	printf ("Q Quits SAOimage\n");
 	printf ("R Raises all SAOimage windows to the top\n");
@@ -779,4 +792,8 @@ PrintFITSHead (header)
  * Aug 20 1999	DJM - Add WCS string argument to wcscom(); decouple X Y
  * Aug 20 1999	DJM - Add coordinate return from print_center()
  * Aug 23 1999	DJM - Fix bug in oldxpix,oldypix saving
+ *
+ * Apr 23 2002	DJM - Fix bug so distance in arcsec really is
+ *
+ * Sep 24 2003	DJM - Improve V command to print dRA and dDec
  */
