@@ -4,7 +4,7 @@ static char SccsId[] = "%W%  %G%";
 
 /* Module:	readiraf.c (Read IRAF)
  * Purpose:	Read "old style" IRAF OIF (.imh) files directly
- * Copyright:	1998 Smithsonian Astrophysical Observatory
+ * Copyright:	2000 Smithsonian Astrophysical Observatory
  *		You may do anything you like with this file except remove
  *		this copyright.  The Smithsonian Astrophysical Observatory
  *		makes no representations about the suitability of this
@@ -152,7 +152,7 @@ int init_irafimh ( img )
   hgeti4 (fitsheader, "NAXIS3", &img->filenimg);
 
   /* Check for existence and open pix file */
-  hgets (fitsheader, "PIXFILE", 256, pixpath); 
+  hgetm (fitsheader, "PIXFIL", 256, pixpath); 
   if ((fd = open_disk(pixpath, IOP_Read, 0)) <= 2 ) {
     pixfile = strrchr (pixpath,'/');
     if (pixfile == NULL || (fd = open_disk(pixfile+1, IOP_Read, 0)) <= 2 ) {
@@ -213,4 +213,6 @@ int init_irafimh ( img )
  * Nov 16 1998	Use utility subroutines in imhfile.c
  * Nov 17 1998	Use separate flags for header and pixfile byte order
  * Nov 18 1998	Read correct number of header bytes
+ *
+ * Mar 28 2000	Read up to 256-char pixel file pathname from multiple keywords
  */
