@@ -1,16 +1,43 @@
 /*** File libwcs/hget.c
- *** April 5, 1999
- *** By Doug Mink, Harvard-Smithsonian Center for Astrophysics
+ *** August 30, 2002
+ *** By Doug Mink, dmink@cfa.harvard.edu
+ *** Harvard-Smithsonian Center for Astrophysics
+ *** Copyright (C) 1994-2002
+ *** Smithsonian Astrophysical Observatory, Cambridge, MA, USA
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+    
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+    Correspondence concerning WCSTools should be addressed as follows:
+           Internet email: dmink@cfa.harvard.edu
+           Postal address: Doug Mink
+                           Smithsonian Astrophysical Observatory
+                           60 Garden St.
+                           Cambridge, MA 02138 USA
 
  * Module:	hget.c (Get FITS Header parameter values)
  * Purpose:	Extract values for variables from FITS header string
  * Subroutine:	hgeti2 (hstring,keyword,ival) returns short integer
+ * Subroutine:	hgeti4c (hstring,keyword,wchar,ival) returns long integer
  * Subroutine:	hgeti4 (hstring,keyword,ival) returns long integer
  * Subroutine:	hgetr4 (hstring,keyword,rval) returns real
  * Subroutine:	hgetra (hstring,keyword,ra) returns double RA in degrees
  * Subroutine:	hgetdec (hstring,keyword,dec) returns double Dec in degrees
+ * Subroutine:	hgetr8c (hstring,keyword,wchar,dval) returns double
  * Subroutine:	hgetr8 (hstring,keyword,dval) returns double
  * Subroutine:	hgetl  (hstring,keyword,lval) returns logical int (0=F, 1=T)
+ * Subroutine:	hgetsc (hstring,keyword,wchar,lstr,str) returns character string
  * Subroutine:	hgets  (hstring,keyword, lstr, str) returns character string
  * Subroutine:	hgetm  (hstring,keyword, lstr, str) returns multi-keyword string
  * Subroutine:	hgetdate (hstring,keyword,date) returns date as fractional year
@@ -24,27 +51,20 @@
  * Subroutine:	strsrch (s1, s2) finds string s2 in null-terminated string s1
  * Subroutine:	strnsrch (s1, s2, ls1) finds string s2 in ls1-byte string s1
  * Subroutine:	hlength (header,lhead) sets length of FITS header for searching
- * Subroutine:  isnum (string) returns 1 if number, else 0
+ * Subroutine:  isnum (string) returns 1 if integer, 2 if fp number, else 0
  * Subroutine:  notnum (string) returns 0 if number, else 1
-
- * Copyright:   1995-1998 Smithsonian Astrophysical Observatory
- *              You may do anything you like with this file except remove
- *              this copyright.  The Smithsonian Astrophysical Observatory
- *              makes no representations about the suitability of this
- *              software for any purpose.  It is provided "as is" without
- *              express or implied warranty.
-
-*/
+ * Subroutine:  numdec (string) returns number of decimal places in numeric string
+ */
 
 #include <string.h>		/* NULL, strlen, strstr, strcpy */
 #include <stdio.h>
 #include "fitshead.h"	/* FITS header extraction subroutines */
 #include <stdlib.h>
 #ifndef VMS
-#include <values.h>
+#include <limits.h>
 #else
-#define MAXINT  2147483647 /* Biggest number that can fit in long */
-#define MAXSHORT 32767
+#define INT_MAX  2147483647 /* Biggest number that can fit in long */
+#define SHRT_MAX 32767
 #endif
 #define VLENGTH 81
 
@@ -54,12 +74,12 @@ static int use_saolib=0;
 
 char *hgetc ();
 
-char val[VLENGTH+1];
+static char val[VLENGTH+1];
+static int multiline = 0;
 
+static int lhead0 = 0;	/* Length of header string */
 
 /* Set the length of the header string, if not terminated by NULL */
-
-static int lhead0 = 0;
 int
 hlength (header, lhead)
 char	*header; /* FITS header */
@@ -74,6 +94,47 @@ int	lhead;	/* Maximum length of FITS header */
     return (lhead0);
 }
 
+/* Return the length of the header string, computing it if lhead0 not set */
+int
+gethlength (header)
+char	*header; /* FITS header */
+{
+    if (lhead0 > 0)
+	return (lhead0);
+    else
+	return (hlength (header, 0));
+}
+
+
+/* Extract Integer*4 value for variable from FITS header string */
+
+int
+hgeti4c (hstring,keyword,wchar,ival)
+
+char	*hstring;	/* character string containing FITS header information
+			   in the format <keyword>= <value> {/ <comment>} */
+char	*keyword;	/* character string containing the name of the keyword
+			   the value of which is returned.  hget searches for
+			   a line beginning with this string.  if "[n]" is
+			   present, the n'th token in the value is returned.
+			   (the first 8 characters must be unique) */
+unsigned char wchar;	/* Character of multiple WCS header; =0 if unused */
+int	*ival;		/* Keyword value returned */
+{
+    char keyword1[16];
+    int lkey;
+
+    if (wchar < 64)
+	return (hgeti4 (hstring, keyword, ival));
+    else {
+	strcpy (keyword1, keyword);
+	lkey = strlen (keyword);
+	keyword1[lkey] = wchar;
+	keyword1[lkey+1] = (char) 0;
+	return (hgeti4 (hstring, keyword1, ival));
+	}
+}
+
 
 /* Extract long value for variable from FITS header string */
 
@@ -99,7 +160,8 @@ int *ival;
 
     /* Translate value from ASCII to binary */
     if (value != NULL) {
-	minint = -MAXINT - 1;
+	if (value[0] == '#') value++;
+	minint = -INT_MAX - 1;
 	lval = strlen (value);
 	if (lval > VLENGTH) {
 	    strncpy (val, value, VLENGTH);
@@ -108,8 +170,8 @@ int *ival;
 	else
 	    strcpy (val, value);
 	dval = atof (val);
-	if (dval+0.001 > MAXINT)
-	    *ival = MAXINT;
+	if (dval+0.001 > INT_MAX)
+	    *ival = INT_MAX;
 	else if (dval >= 0)
 	    *ival = (int) (dval + 0.001);
 	else if (dval-0.001 < minint)
@@ -148,6 +210,7 @@ short *ival;
 
     /* Translate value from ASCII to binary */
     if (value != NULL) {
+	if (value[0] == '#') value++;
 	lval = strlen (value);
 	if (lval > VLENGTH) {
 	    strncpy (val, value, VLENGTH);
@@ -156,9 +219,9 @@ short *ival;
 	else
 	    strcpy (val, value);
 	dval = atof (val);
-	minshort = -MAXSHORT - 1;
-	if (dval+0.001 > MAXSHORT)
-	    *ival = MAXSHORT;
+	minshort = -SHRT_MAX - 1;
+	if (dval+0.001 > SHRT_MAX)
+	    *ival = SHRT_MAX;
 	else if (dval >= 0)
 	    *ival = (short) (dval + 0.001);
 	else if (dval-0.001 < minshort)
@@ -194,6 +257,7 @@ float *rval;
 
     /* translate value from ASCII to binary */
     if (value != NULL) {
+	if (value[0] == '#') value++;
 	lval = strlen (value);
 	if (lval > VLENGTH) {
 	    strncpy (val, value, VLENGTH);
@@ -268,6 +332,37 @@ double *dval;	/* Right ascension in degrees (returned) */
 }
 
 
+/* Extract real*8 value for variable from FITS header string */
+
+int
+hgetr8c (hstring,keyword,wchar,dval)
+
+char	*hstring;	/* character string containing FITS header information
+			   in the format <keyword>= <value> {/ <comment>} */
+char	*keyword;	/* character string containing the name of the keyword
+			   the value of which is returned.  hget searches for
+			   a line beginning with this string.  if "[n]" is
+			   present, the n'th token in the value is returned.
+			   (the first 8 characters must be unique) */
+unsigned char wchar;	/* Character of multiple WCS header; =0 if unused */
+double	*dval;		/* Keyword value returned */
+{
+    char keyword1[16];
+    int lkey;
+
+    if (wchar < 64)
+	return (hgetr8 (hstring, keyword, dval));
+    else {
+	strcpy (keyword1, keyword);
+	lkey = strlen (keyword);
+	keyword1[lkey] = wchar;
+	keyword1[lkey+1] = (char) 0;
+	return (hgetr8 (hstring, keyword1, dval));
+	}
+}
+
+
+
 /* Extract real*8 value for variable from FITS header string */
 
 int
@@ -290,6 +385,7 @@ double *dval;
 
     /* Translate value from ASCII to binary */
     if (value != NULL) {
+	if (value[0] == '#') value++;
 	lval = strlen (value);
 	if (lval > VLENGTH) {
 	    strncpy (val, value, VLENGTH);
@@ -380,6 +476,7 @@ double *dval;
 	if (sstr > value) {
 	    *sstr = '\0';
 	    day = (int) atof (value);
+	    *sstr = '/';
 	    nval = sstr + 1;
 	    sstr = strchr (nval,'/');
 	    if (sstr == NULL)
@@ -387,8 +484,14 @@ double *dval;
 	    if (sstr > value) {
 		*sstr = '\0';
 		month = (int) atof (nval);
+		*sstr = '/';
 		nval = sstr + 1;
 		year = (int) atof (nval);
+		if (day > 31) {
+		    yday = year;
+		    year = day;
+		    day = yday;
+		    }
 		if (year >= 0 && year <= 49)
 		    year = year + 2000;
 		else if (year < 100)
@@ -421,6 +524,7 @@ double *dval;
 	else if (dstr > value) {
 	    *dstr = '\0';
 	    year = (int) atof (value);
+	    *dstr = '-';
 	    nval = dstr + 1;
 	    dstr = strchr (nval,'-');
 	    month = 1;
@@ -429,11 +533,14 @@ double *dval;
 	    if (dstr > value) {
 		*dstr = '\0';
 		month = (int) atof (nval);
+		*dstr = '-';
 		nval = dstr + 1;
 		tstr = strchr (nval,'T');
 		if (tstr > value)
 		    *tstr = '\0';
 		day = (int) atof (nval);
+		if (tstr > value)
+		    *tstr = 'T';
 		}
 
 	    /* If year is < 32, it is really day of month in old format */
@@ -472,14 +579,19 @@ double *dval;
 		if (cstr > value) {
 		    *cstr = '\0';
 		    hours = (int) atof (nval);
+		    *cstr = ':';
 		    nval = cstr + 1;
 		    cstr = strchr (nval,':');
 		    if (cstr > value) {
+			*cstr = '\0';
 			minutes = (int) atof (nval);
+			*cstr = ':';
 			nval = cstr + 1;
-			cstr = strchr (nval,':');
-			if (cstr > value)
-			    seconds = atof (nval);
+			seconds = atof (nval);
+			}
+		    else {
+			minutes = (int) atof (nval);
+			seconds = 0.0;
 			}
 		    }
 		fday = ((3.6e3 * (double)hours) + (6.e1 * (double)minutes) +
@@ -515,13 +627,31 @@ char *str;	/* String (returned) */
     char *stri;
     char keywordi[16];
     int lval, lstri, ikey;
+    char keyform[8];
 
     stri = str;
     lstri = lstr;
 
+    sprintf (keywordi, "%s_1", keyword);
+    if (ksearch (hstring, keywordi))
+	strcpy (keyform, "%s_%d");
+    else {
+	sprintf (keywordi, "%s_01", keyword);
+	if (ksearch (hstring, keywordi))
+	    strcpy (keyform, "%s_%02d");
+	else {
+	    sprintf (keywordi, "%s_001", keyword);
+	    if (ksearch (hstring, keywordi))
+		strcpy (keyform, "%s_%03d");
+	    else
+		return (0);
+	    }
+	}
+
     /* Loop through sequentially-named keywords */
+    multiline = 1;
     for (ikey = 1; ikey < 20; ikey++) {
-	sprintf (keywordi, "%s_%03d", keyword, ikey);
+	sprintf (keywordi, keyform, keyword, ikey);
 
 	/* Get value for this keyword */
 	value = hgetc (hstring, keywordi);
@@ -531,6 +661,7 @@ char *str;	/* String (returned) */
 		strcpy (stri, value);
 	    else if (lstri > 1) {
 		strncpy (stri, value, lstri-1);
+		stri[lstri] = (char) 0;
 		break;
 		}
 	    else {
@@ -543,6 +674,7 @@ char *str;	/* String (returned) */
 	stri = stri + lval;
 	lstri = lstri - lval;
 	}
+    multiline = 0;
 
     /* Return 1 if any keyword found, else 0 */
     if (ikey > 1)
@@ -552,6 +684,37 @@ char *str;	/* String (returned) */
 }
 
 
+/* Extract string value for variable from FITS header string */
+
+int
+hgetsc (hstring,keyword,wchar,lstr,str)
+
+char	*hstring;	/* character string containing FITS header information
+			   in the format <keyword>= <value> {/ <comment>} */
+char	*keyword;	/* character string containing the name of the keyword
+			   the value of which is returned.  hget searches for
+			   a line beginning with this string.  if "[n]" is
+			   present, the n'th token in the value is returned.
+			   (the first 8 characters must be unique) */
+unsigned char wchar;	/* Character of multiple WCS header; =0 if unused */
+int	lstr;		/* Size of str in characters */
+char	*str;		/* String (returned) */
+{
+    char keyword1[16];
+    int lkey;
+
+    if (wchar < 64)
+	return (hgets (hstring, keyword, lstr, str));
+    else {
+	strcpy (keyword1, keyword);
+	lkey = strlen (keyword);
+	keyword1[lkey] = wchar;
+	keyword1[lkey+1] = (char) 0;
+	return (hgets (hstring, keyword1, lstr, str));
+	}
+}
+
+
 /* Extract string value for variable from FITS header string */
 
 int
@@ -602,7 +765,7 @@ char *keyword;	/* character string containing the name of the keyword
 		   (the first 8 characters must be unique) */
 int *ndec;	/* Number of decimal places in keyword value */
 {
-    char *value,val[30];
+    char *value;
     int i, nchar;
 
     /* Get value and comment from header string */
@@ -617,6 +780,7 @@ int *ndec;	/* Number of decimal places in keyword value */
 		return (1);
 	    *ndec = *ndec + 1;
 	    }
+	return (1);
 	}
     else
 	return (0);
@@ -723,11 +887,24 @@ char *keyword0;	/* character string containing the name of the keyword
     /* Extract value and remove excess spaces */
     if (q1 != NULL) {
 	v1 = q1 + 1;;
-	v2 = q2;
-	c1 = strsrch (q2,"/");
+	if (q2 != NULL) {
+	    v2 = q2;
+	    c1 = strsrch (q2,"/");
+	    }
+	else {
+	    c1 = strsrch (line,"/");
+	    if (c1 != NULL)
+		v2 = c1;
+	    else
+		v2 = line + 79;
+	    }
 	}
     else {
-	v1 = strsrch (line,"=") + 1;
+	v1 = strsrch (line,"=");
+	if (v1 == NULL)
+	    v1 = line + 9;
+	else
+	    v1 = v1 + 1;
 	c1 = strsrch (line,"/");
 	if (c1 != NULL)
 	    v2 = c1;
@@ -735,19 +912,24 @@ char *keyword0;	/* character string containing the name of the keyword
 	    v2 = line + 79;
 	}
 
-    /* Ignore leading spaces */
-    while (*v1 == ' ' && v1 < v2) {
-	v1++;
+    /* Ignore leading spaces if not multiline */
+    if (!multiline) {
+	while (*v1 == ' ' && v1 < v2) {
+	    v1++;
+	    }
 	}
 
     /* Drop trailing spaces */
     *v2 = '\0';
-    v2--;
-    while ((*v2 == ' ' || *v2 == (char) 13) && v2 > v1) {
-	*v2 = '\0';
+    if (!multiline) {
 	v2--;
+	while ((*v2 == ' ' || *v2 == (char) 13) && v2 > v1) {
+	    *v2 = '\0';
+	    v2--;
+	    }
 	}
 
+    /* Convert -zero to just plain 0 */
     if (!strcmp (v1, "-0"))
 	v1++;
     strcpy (cval,v1);
@@ -759,15 +941,34 @@ char *keyword0;	/* character string containing the name of the keyword
 	if (brack2 != NULL)
 	    *brack2 = '\0';
 	ipar = atoi (brack1);
+	cwhite[0] = ' ';
+	cwhite[1] = '\0';
 	if (ipar > 0) {
-	    cwhite[0] = ' ';
-	    cwhite[1] = '\0';
 	    for (i = 1; i <= ipar; i++) {
 		cpar = strtok (v1,cwhite);
 		v1 = NULL;
 		}
-	    if (cpar != NULL)
+	    if (cpar != NULL) {
 		strcpy (cval,cpar);
+		value = cval;
+		}
+	    else
+		value = NULL;
+	    }
+
+	/* If token counter is negative, include rest of value from token on */
+	else if (ipar < 0) {
+	    for (i = 1; i < -ipar; i++) {
+		v1 = strchr (v1, ' ');
+		if (v1 == NULL)
+		    break;
+		else
+		    v1 = v1 + 1;
+		}
+	    if (v1 != NULL) {
+		strcpy (cval, v1);
+		value = cval;
+		}
 	    else
 		value = NULL;
 	    }
@@ -813,7 +1014,7 @@ char *keyword;	/* character string containing the name of the variable
 	lhstr = lhead0;
     else {
 	lhstr = 0;
-	while (lhstr < 57600 && hstring[lhstr] != 0)
+	while (lhstr < 256000 && hstring[lhstr] != 0)
 	    lhstr++;
 	}
     headlast = hstring + lhstr;
@@ -880,7 +1081,8 @@ char *keyword;	/* character string containing the name of the variable
 
 /* Find FITS header line containing specified keyword */
 
-char *ksearch (hstring,keyword)
+char *
+ksearch (hstring,keyword)
 
 /* Find entry for keyword keyword in FITS header string hstring.
    (the keyword may have a maximum of eight letters)
@@ -898,7 +1100,7 @@ char *keyword;	/* character string containing the name of the variable
 		or '$'.  it is truncated to 8 characters. */
 {
     char *loc, *headnext, *headlast, *pval, *lc, *line;
-    int icol, nextchar, lkey, nleft, lhstr;
+    int icol, nextchar, lkey, nleft, lhead, lmax;
 
 #ifdef USE_SAOLIB
 	int iel=1, ip=1, nel, np, ier;
@@ -909,15 +1111,18 @@ char *keyword;	/* character string containing the name of the variable
 
     pval = 0;
 
-/* Search header string for variable name */
+/* Find current length of header string */
     if (lhead0)
-	lhstr = lhead0;
-    else {
-	lhstr = 0;
-	while (lhstr < 57600 && hstring[lhstr] != 0)
-	    lhstr++;
+	lmax = lhead0;
+    else
+	lmax = 256000;
+    for (lhead = 0; lhead < lmax; lhead++) {
+	if (hstring[lhead] == (char) 0)
+	    break;
 	}
-    headlast = hstring + lhstr;
+
+/* Search header string for variable name */
+    headlast = hstring + lhead;
     headnext = hstring;
     pval = NULL;
     while (headnext < headlast) {
@@ -1000,29 +1205,54 @@ char	*in;	/* Character string of sexigesimal or decimal degrees */
 {
     double dec;		/* Declination in degrees (returned) */
     double deg, min, sec, sign;
-    char *value, *c1;
+    char *value, *c1, *c2;
+    int lval;
 
     dec = 0.0;
 
+    /* Return 0.0 if string is null */
+    if (in == NULL)
+	return (dec);
+
     /* Translate value from ASCII colon-delimited string to binary */
     if (in[0]) {
 	value = in;
-	if (!strsrch (value,"-"))
-	    sign = 1.0;
-	else {
+
+	/* Remove leading spaces */
+	while (*value == ' ')
+	    value++;
+
+	/* Save sign */
+	if (*value == '-') {
 	    sign = -1.0;
-	    value = strsrch (value,"-") + 1;
+	    value++;
 	    }
-	if ((c1 = strsrch (value,":")) != NULL) {
+	else if (*value == '+') {
+	    sign = 1.0;
+	    value++;
+	    }
+	else
+	    sign = 1.0;
+
+	/* Remove trailing spaces */
+	lval = strlen (value);
+	while (value[lval-1] == ' ')
+	    lval--;
+	
+	if ((c1 = strsrch (value,":")) == NULL)
+	    c1 = strnsrch (value," ",lval);
+	if (c1 != NULL) {
 	    *c1 = 0;
 	    deg = (double) atoi (value);
 	    *c1 = ':';
 	    value = c1 + 1;
-	    if ((c1 = strsrch (value,":")) != NULL) {
-		*c1 = 0;
+	    if ((c2 = strsrch (value,":")) == NULL)
+		c2 = strsrch (value," ");
+	    if (c2 != NULL) {
+		*c2 = 0;
 		min = (double) atoi (value);
-		*c1 = ':';
-		value = c1 + 1;
+		*c2 = ':';
+		value = c2 + 1;
 		sec = atof (value);
 		}
 	    else {
@@ -1121,6 +1351,124 @@ int	ls1;	/* Length of string being searched */
 }
 
 
+/* Find string s2 within null-terminated string s1 (case-free search) */
+
+char *
+strcsrch (s1, s2)
+
+char *s1;	/* String to search */
+char *s2;	/* String to look for */
+
+{
+    int ls1;
+    ls1 = strlen (s1);
+    return (strncsrch (s1, s2, ls1));
+}
+
+
+/* Find string s2 within string s1 (case-free search) */
+
+char *
+strncsrch (s1, s2, ls1)
+
+char	*s1;	/* String to search */
+char	*s2;	/* String to look for */
+int	ls1;	/* Length of string being searched */
+
+{
+    char *s,*s1e, sl, *os2;
+    char cfirst,clast,ocfirst,oclast;
+    int i,ls2;
+
+    /* Return null string if either pointer is NULL */
+    if (s1 == NULL || s2 == NULL)
+	return (NULL);
+
+    /* A zero-length pattern is found in any string */
+    ls2 = strlen (s2);
+    if (ls2 ==0)
+	return (s1);
+
+    /* Only a zero-length string can be found in a zero-length string */
+    if (ls1 ==0)
+	return (NULL);
+
+    /* For one or two characters, set opposite case first and last letters */
+    if (ls2 < 3) {
+	cfirst = s2[0];
+	if (cfirst > 96 && cfirst < 123)
+	    ocfirst = cfirst - 32;
+	else if (cfirst > 64 && cfirst < 91)
+	    ocfirst = cfirst + 32;
+	else
+	    ocfirst = cfirst;
+	if (ls2 > 1) {
+	    clast = s2[1];
+	    if (clast > 96 && clast < 123)
+		oclast = clast - 32;
+	    else if (clast > 64 && clast < 91)
+		oclast = clast + 32;
+	    else
+		oclast = clast;
+	    }
+	}
+
+    /* Else duplicate string with opposite case letters for comparison */
+    else {
+	os2 = (char *) calloc (ls2, 1);
+	for (i = 0; i < ls2; i++) {
+	    if (s2[i] > 96 && s2[i] < 123)
+		os2[i] = s2[i] - 32;
+	    else if (s2[i] > 64 && s2[i] < 91)
+		os2[i] = s2[i] + 32;
+	    else
+		os2[i] = s2[i];
+	    }
+	cfirst = s2[0];
+	ocfirst = os2[0];
+	clast = s2[ls2-1];
+	oclast = os2[ls2-1];
+	}
+
+    /* Loop through input string, character by character */
+    s1e = s1 + ls1 - ls2 + 1;
+    s = s1;
+    while (s < s1e) { 
+
+	/* Search for first character in pattern string */
+	if (*s == cfirst || *s == ocfirst) {
+
+	    /* If single character search, return */
+	    if (ls2 == 1)
+		return (s);
+
+	    /* Search for last character in pattern string if first found */
+	    sl = s[ls2-1];
+	    if (sl == clast || sl == oclast) {
+
+		/* If two-character search, return */
+		if (ls2 == 2)
+		    return (s);
+
+		/* If 3 or more characters, check for rest of search string */
+		i = 1;
+		while (i < ls2 && (s[i] == s2[i] || s[i] == os2[i]))
+		    i++;
+
+		/* If entire string matches, return */
+		if (i >= ls2) {
+		    free (os2);
+		    return (s);
+		    }
+		}
+	    }
+	s++;
+	}
+    free (os2);
+    return (NULL);
+}
+
+
 int
 notnum (string)
 
@@ -1133,34 +1481,98 @@ char *string;	/* Character string */
 }
 
 
+/* ISNUM-- Return 1 if string is an integer number, 2 if floating point, else 0
+ */
+
 int
 isnum (string)
 
 char *string;	/* Character string */
 {
     int lstr, i, nd;
-    char cstr;
+    char cstr, cstr1;
+    int fpcode;
+
+    /* Return 0 if string is NULL */
+    if (string == NULL)
+	return (0);
 
     lstr = strlen (string);
     nd = 0;
+    fpcode = 1;
+
+    /* Return 0 if string starts with a D or E */
+    cstr = string[0];
+    if (cstr == 'D' || cstr == 'd' ||
+	cstr == 'E' || cstr == 'e') {
+	return (0);
+	}
+
+    /* Remove trailing spaces */
+    while (string[lstr-1] == ' ')
+	lstr--;
+
+    /* Numeric strings contain 0123456789-+ and d or e for exponents */
     for (i = 0; i < lstr; i++) {
 	cstr = string[i];
+	if (cstr == '\n')
+	    break;
+
+	/* Ignore leading spaces */
+	if (cstr == ' ' && nd == 0)
+	    continue;
+
 	if ((cstr < 48 || cstr > 57) &&
 	    cstr != '+' && cstr != '-' &&
 	    cstr != 'D' && cstr != 'd' &&
 	    cstr != 'E' && cstr != 'e' &&
 	    cstr != '.')
 	    return (0);
+	else if (cstr == '+' || cstr == '-') {
+	    if (string[i+1] == '-' || string[i+1] == '+')
+		return (0);
+	    else if (i > 0) {
+		cstr1 = string[i-1];
+		if (cstr1 != 'D' && cstr1 != 'd' &&
+		    cstr1 != 'E' && cstr1 != 'e' &&
+		    cstr1 != ' ')
+		    return (0);
+		}
+	    }
 	else if (cstr >= 47 && cstr <= 57)
 	    nd++;
+	if (cstr=='.' || cstr=='d' || cstr=='e' || cstr=='d' || cstr=='e')
+	    fpcode = 2;
 	}
     if (nd > 0)
-	return (1);
+	return (fpcode);
     else
 	return (0);
 }
 
 
+/* NUMDEC -- Return number of decimal places in numeric string (-1 if not number) */
+
+int
+numdec (string)
+
+char *string;   /* Numeric string */
+{
+    char *cdot;
+    int lstr;
+
+    if (notnum (string))
+        return (-1);
+    else {
+        lstr = strlen (string);
+        if ((cdot = strchr (string, '.')) == NULL)
+            return (0);
+        else
+            return (lstr - (cdot - string));
+        }
+}
+
+
 #ifdef USE_SAOLIB
 int set_saolib(hstring)
     void *hstring;
@@ -1221,4 +1633,38 @@ int set_saolib(hstring)
  * Oct  7 1998	Clean up search for last blank line
  *
  * Apr  5 1999	Check lengths of strings before copying them
+ * May  5 1999	values.h -> POSIX limits.h: MAXINT->INT_MAX, MAXSHORT->SHRT_MAX
+ * Jul 15 1999	Add hgetm() options of 1- or 2-digit keyword extensions
+ * Oct  6 1999	Add gethlength() to return header length
+ * Oct 14 1999	In ksearch(), search only to null not to end of buffer
+ * Oct 15 1999	Return 1 from hgetndec() if successful
+ * Oct 20 1999	Drop unused variable after lint (val in hgetndec)
+ * Dec  3 1999	Fix isnum() to reject strings starting with a d or e
+ * Dec 20 1999	Update hgetdate() to get minutes and seconds right
+ *
+ * Feb 10 2000	Parse RA and Dec with spaces as well as colons as separators
+ * Feb 11 2000	Add null at end of multi-line keyword value character string
+ * Feb 25 2000	Change max search string length from 57600 to 256000
+ * Mar 15 2000	Deal with missing second quotes in string values
+ * Mar 17 2000	Return 2 from isnum() if number is floating point (.de)
+ * Mar 17 2000	Ignore leading # for numeric values in header
+ * Mar 21 2000	Implement -n to get string value starting with nth token
+ * Apr  5 2000	Reject +- in isnum()
+ * Jun  9 2000	Read keyword values even if no equal sign is present
+ * Sep 20 2000	Ignore linefeed at end of number in isnum()
+ * Oct 23 2000	Fix handling of embedded + or - in isnum()
+ *
+ * Jan 19 2000	Return 0 from isnum(), str2ra(), and str2dec() if string is null
+ * Mar 30 2001	Fix header length finding algorithm in ksearch()
+ * Jul 13 2001	Make val[] static int instead of int; drop unused variables
+ * Sep 12 2001	Read yyyy/mm/dd dates as well as dd/mm/yyyy
+ * Sep 20 2001	Ignore leading spaces in str2dec()
+ * Sep 20 2001	Ignore trailing spaces in isnum()
+ *
+ * Apr  3 2002	Add hgetr8c(), hgeti4c(), and hgetsc() for multiple WCS handling
+ * Apr 26 2002	Fix bug in hgetsc(), hgeti4c(), and hgetr8c() found by Bill Joye
+ * Jun 26 2002	Do not drop leading or trailing spaces in multi-line values
+ * Aug  6 2002	Add strcsrch() and strncsrch() for case-insensitive searches
+ * Aug 30 2002	Fix bug so strcsrch() really is case-insensitive
+ * Oct 20 2003	Add numdec() to return number of decimal places in a string
  */
