@@ -1,6 +1,30 @@
 /*** File libwcs/wcs.c
- *** April 7, 1999
- *** By Doug Mink, Harvard-Smithsonian Center for Astrophysics
+ *** November 3, 2003
+ *** By Doug Mink, dmink@cfa.harvard.edu
+ *** Harvard-Smithsonian Center for Astrophysics
+ *** Copyright (C) 1994-2003
+ *** Smithsonian Astrophysical Observatory, Cambridge, MA, USA
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+    
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+    Correspondence concerning WCSTools should be addressed as follows:
+           Internet email: dmink@cfa.harvard.edu
+           Postal address: Doug Mink
+                           Smithsonian Astrophysical Observatory
+                           60 Garden St.
+                           Cambridge, MA 02138 USA
 
  * Module:	wcs.c (World Coordinate Systems)
  * Purpose:	Convert FITS WCS to pixels and vice versa:
@@ -15,15 +39,16 @@
  * Subroutine:	wcscdset (wcs, cd) sets rotation and scaling from a CD matrix
  * Subroutine:	wcspcset (wcs,cdelt1,cdelt2,pc) sets rotation and scaling
  * Subroutine:	wcseqset (wcs, equinox) resets an existing WCS structure to new equinox
- * Subroutine:	setdefwcs (oldwcs) set flag to use AIPS WCS subroutines
- * Subroutine:	getdefwcs (oldwcs) get value of flag to use AIPS WCS subroutines
  * Subroutine:	iswcs(wcs) returns 1 if WCS structure is filled, else 0
  * Subroutine:	nowcs(wcs) returns 0 if WCS structure is filled, else 1
  * Subroutine:	wcscent (wcs) prints the image center and size in WCS units
  * Subroutine:	wcssize (wcs, cra, cdec, dra, ddec) returns image center and size
  * Subroutine:	wcsfull (wcs, cra, cdec, width, height) returns image center and size
+ * Subroutine:	wcsrange (wcs, ra1, ra2, dec1, dec2) returns image coordinate limits
+
  * Subroutine:	wcsshift (wcs,cra,cdec) resets the center of a WCS structure
  * Subroutine:	wcsdist (x1,y1,x2,y2) compute angular distance between ra/dec or lat/long
+ * Subroutine:	wcsdiff (x1,y1,x2,y2) compute angular distance between ra/dec or lat/long
  * Subroutine:	wcscominit (wcs,command) sets up a command format for execution by wcscom
  * Subroutine:	wcsoutinit (wcs,coor) sets up the coordinate system used by pix2wcs
  * Subroutine:	getwcsout (wcs) returns current output coordinate system used by pix2wcs
@@ -31,7 +56,7 @@
  * Subroutine:	getwcsin (wcs) returns current input coordinate system used by wcs2pix
  * Subroutine:	setwcsdeg(wcs, new) sets WCS output in degrees or hh:mm:ss
  * Subroutine:	getradecsys(wcs) returns current coordinate system type
- * Subroutine:	wcscom (wcs,file,x,y) executes a command using the current world coordinates
+ * Subroutine:	wcscom (wcs,file,x,y,wcstr) executes a command using the current world coordinates
  * Subroutine:	setwcslin (wcs, mode) sets output string mode for LINEAR
  * Subroutine:	pix2wcst (wcs,xpix,ypix,wcstring,lstr) pixels -> sky coordinate string
  * Subroutine:	pix2wcs (wcs,xpix,ypix,xpos,ypos) pixel coordinates -> sky coordinates
@@ -42,29 +67,21 @@
  * Subroutine:	setwcsfile (filename)  Set file name for error messages 
  * Subroutine:	setwcserr (errmsg)  Set error message 
  * Subroutine:	wcserr()  Print error message 
- * Subroutine:	setdefwcs (oldwcs)  Set flag to choose AIPS or WCSLIB WCS subroutines 
+ * Subroutine:	setdefwcs (wcsproj)  Set flag to choose AIPS or WCSLIB WCS subroutines 
  * Subroutine:	getdefwcs()  Get flag to switch between AIPS and WCSLIB subroutines 
  * Subroutine:	savewcscoor (wcscoor)
  * Subroutine:	getwcscoor()  Return preset output default coordinate system 
  * Subroutine:	savewcscom (i, wcscom)  Save specified WCS command 
  * Subroutine:	setwcscom (wcs)  Initialize WCS commands 
  * Subroutine:	getwcscom (i)  Return specified WCS command 
+ * Subroutine:	wcsfree (wcs)  Free storage used by WCS structure
  * Subroutine:	freewcscom (wcs)  Free storage used by WCS commands 
-
- * Copyright:   1999 Smithsonian Astrophysical Observatory
- *              You may do anything you like with this file except remove
- *              this copyright.  The Smithsonian Astrophysical Observatory
- *              makes no representations about the suitability of this
- *              software for any purpose.  It is provided "as is" without
- *              express or implied warranty.
-
  */
 
 #include <string.h>		/* strstr, NULL */
 #include <stdio.h>		/* stderr */
 #include <math.h>
 #include "wcs.h"
-#include "fitshead.h"
 #ifndef VMS
 #include <stdlib.h>
 #endif
@@ -73,7 +90,7 @@ static char wcserrmsg[80];
 static char wcsfile[256]={""};
 static void wcslibrot();
 void wcsrotset();
-static int oldwcs0 = 0;
+static int wcsproj0 = 0;
 static int izpix = 0;
 static double zpix = 0.0;
 
@@ -81,7 +98,18 @@ void
 wcsfree (wcs)
 struct WorldCoor *wcs;	/* WCS structure */
 {
+    if (nowcs (wcs)) {
+
+	/* Free WCS structure if allocated but not filled */
+	if (wcs)
+	    free (wcs);
+
+	return;
+	}
+
     freewcscom (wcs);
+    if (wcs->wcsname != NULL)
+	free (wcs->wcsname);
     if (wcs->lin.imgpix != NULL)
 	free (wcs->lin.imgpix);
     if (wcs->lin.piximg != NULL)
@@ -111,8 +139,6 @@ char	*proj;	/* Projection */
 {
     struct WorldCoor *wcs;
     double cdelt1, cdelt2;
-    char *str;
-    char envar[16];
 
     wcs = (struct WorldCoor *) calloc (1, sizeof(struct WorldCoor));
 
@@ -122,12 +148,12 @@ char	*proj;	/* Projection */
     wcs->wcsl.flag = 0;
 
     /* Image dimensions */
-    wcs->naxes = 2;
+    wcs->naxis = 2;
     wcs->lin.naxis = 2;
     wcs->nxpix = nxpix;
     wcs->nypix = nypix;
 
-    wcs->oldwcs = oldwcs0;
+    wcs->wcsproj = wcsproj0;
 
     wcs->crpix[0] = xrpix;
     wcs->crpix[1] = yrpix;
@@ -156,7 +182,7 @@ char	*proj;	/* Projection */
     strcat (wcs->ctype[1],proj);
 
     if (wcstype (wcs, wcs->ctype[0], wcs->ctype[1])) {
-	free (wcs);
+	wcsfree (wcs);
 	return (NULL);
 	}
     
@@ -170,9 +196,9 @@ char	*proj;	/* Projection */
     /* Coordinate reference frame and equinox */
     wcs->equinox =  (double) equinox;
     if (equinox > 1980)
-	strcpy (wcs->radecsys,"J2000");
+	strcpy (wcs->radecsys,"FK5");
     else
-	strcpy (wcs->radecsys,"B1950");
+	strcpy (wcs->radecsys,"FK4");
     if (epoch > 0)
 	wcs->epoch = epoch;
     else
@@ -213,8 +239,6 @@ double	epoch;	/* Epoch of coordinates, used for FK4/FK5 conversion
 		 * no effect if 0 */
 {
     struct WorldCoor *wcs;
-    char *str;
-    char envar[16];
 
     wcs = (struct WorldCoor *) calloc (1, sizeof(struct WorldCoor));
 
@@ -224,12 +248,12 @@ double	epoch;	/* Epoch of coordinates, used for FK4/FK5 conversion
     wcs->wcsl.flag = 0;
 
     /* Image dimensions */
-    wcs->naxes = 2;
+    wcs->naxis = 2;
     wcs->lin.naxis = 2;
     wcs->nxpix = naxis1;
     wcs->nypix = naxis2;
 
-    wcs->oldwcs = oldwcs0;
+    wcs->wcsproj = wcsproj0;
 
     wcs->crpix[0] = crpix1;
     wcs->crpix[1] = crpix2;
@@ -238,7 +262,7 @@ double	epoch;	/* Epoch of coordinates, used for FK4/FK5 conversion
     wcs->lin.crpix = wcs->crpix;
 
     if (wcstype (wcs, ctype1, ctype2)) {
-	free (wcs);
+	wcsfree (wcs);
 	return (NULL);
 	}
     if (wcs->latbase == 90)
@@ -270,9 +294,9 @@ double	epoch;	/* Epoch of coordinates, used for FK4/FK5 conversion
     /* Coordinate reference frame and equinox */
     wcs->equinox =  (double) equinox;
     if (equinox > 1980)
-	strcpy (wcs->radecsys,"J2000");
+	strcpy (wcs->radecsys,"FK5");
     else
-	strcpy (wcs->radecsys,"B1950");
+	strcpy (wcs->radecsys,"FK4");
     if (epoch > 0)
 	wcs->epoch = epoch;
     else
@@ -305,39 +329,46 @@ char	*ctype2;	/* FITS WCS projection for axis 2 */
 
 {
     int i, iproj;
-    int nctype = 30;
-    char ctypes[30][4];
+    int nctype = 32;
+    char ctypes[32][4];
+    char dtypes[10][4];
 
+    /* Initialize projection types */
     strcpy (ctypes[0], "LIN");
     strcpy (ctypes[1], "AZP");
-    strcpy (ctypes[2], "TAN");
-    strcpy (ctypes[3], "SIN");
-    strcpy (ctypes[4], "STG");
-    strcpy (ctypes[5], "ARC");
-    strcpy (ctypes[6], "ZPN");
-    strcpy (ctypes[7], "ZEA");
-    strcpy (ctypes[8], "AIR");
-    strcpy (ctypes[9], "CYP");
-    strcpy (ctypes[10], "CAR");
-    strcpy (ctypes[11], "MER");
-    strcpy (ctypes[12], "CEA");
-    strcpy (ctypes[13], "COP");
-    strcpy (ctypes[14], "COD");
-    strcpy (ctypes[15], "COE");
-    strcpy (ctypes[16], "COO");
-    strcpy (ctypes[17], "BON");
-    strcpy (ctypes[18], "PCO");
-    strcpy (ctypes[19], "GLS");
-    strcpy (ctypes[20], "PAR");
-    strcpy (ctypes[21], "AIT");
-    strcpy (ctypes[22], "MOL");
-    strcpy (ctypes[23], "CSC");
-    strcpy (ctypes[24], "QSC");
-    strcpy (ctypes[25], "TSC");
-    strcpy (ctypes[26], "NCP");
-    strcpy (ctypes[27], "DSS");
-    strcpy (ctypes[28], "PLT");
-    strcpy (ctypes[29], "TNX");
+    strcpy (ctypes[2], "SZP");
+    strcpy (ctypes[3], "TAN");
+    strcpy (ctypes[4], "SIN");
+    strcpy (ctypes[5], "STG");
+    strcpy (ctypes[6], "ARC");
+    strcpy (ctypes[7], "ZPN");
+    strcpy (ctypes[8], "ZEA");
+    strcpy (ctypes[9], "AIR");
+    strcpy (ctypes[10], "CYP");
+    strcpy (ctypes[11], "CAR");
+    strcpy (ctypes[12], "MER");
+    strcpy (ctypes[13], "CEA");
+    strcpy (ctypes[14], "COP");
+    strcpy (ctypes[15], "COD");
+    strcpy (ctypes[16], "COE");
+    strcpy (ctypes[17], "COO");
+    strcpy (ctypes[18], "BON");
+    strcpy (ctypes[19], "PCO");
+    strcpy (ctypes[20], "SFL");
+    strcpy (ctypes[21], "PAR");
+    strcpy (ctypes[22], "AIT");
+    strcpy (ctypes[23], "MOL");
+    strcpy (ctypes[24], "CSC");
+    strcpy (ctypes[25], "QSC");
+    strcpy (ctypes[26], "TSC");
+    strcpy (ctypes[27], "NCP");
+    strcpy (ctypes[28], "GLS");
+    strcpy (ctypes[29], "DSS");
+    strcpy (ctypes[30], "PLT");
+    strcpy (ctypes[31], "TNX");
+
+    /* Initialize distortion types */
+    strcpy (dtypes[1], "SIP");
 
     if (!strncmp (ctype1, "LONG",4))
 	strncpy (ctype1, "XLON",4);
@@ -393,39 +424,59 @@ char	*ctype2;	/* FITS WCS projection for axis 2 */
 		wcs->prjcode = i;
 	    }
 
-	/* Handle obsolete projection */
-	if (wcs->prjcode == WCS_NCP)
-	    wcs->oldwcs = 1;
+	/* Handle "obsolete" NCP projection (now WCSLIB should be OK)
+	if (wcs->prjcode == WCS_NCP) {
+	    if (wcs->wcsproj == WCS_BEST)
+		wcs->wcsproj = WCS_OLD;
+	    else if (wcs->wcsproj == WCS_ALT)
+		wcs->wcsproj = WCS_NEW;
+	    } */
 
-	/* Work around bug in WCSLIB handling of CAR projection */
-	if (wcs->prjcode == WCS_CAR)
-	    wcs->oldwcs = 1;
+	/* Work around bug in WCSLIB handling of CAR projection
+	else if (wcs->prjcode == WCS_CAR) {
+	    if (wcs->wcsproj == WCS_BEST)
+		wcs->wcsproj = WCS_OLD;
+	    else if (wcs->wcsproj == WCS_ALT)
+		wcs->wcsproj = WCS_NEW;
+	    } */
 
-	/* Work around bug in WCSLIB handling of COE projection */
-	if (wcs->prjcode == WCS_COE)
-	    wcs->oldwcs = 1;
+	/* Work around bug in WCSLIB handling of COE projection
+	else if (wcs->prjcode == WCS_COE) {
+	    if (wcs->wcsproj == WCS_BEST)
+		wcs->wcsproj = WCS_OLD;
+	    else if (wcs->wcsproj == WCS_ALT)
+		wcs->wcsproj = WCS_NEW;
+	    }
 
-	if (wcs->oldwcs && (
+	else if (wcs->wcsproj == WCS_BEST) */
+	if (wcs->wcsproj == WCS_BEST)
+	    wcs->wcsproj = WCS_NEW;
+
+	else if (wcs->wcsproj == WCS_ALT)
+	    wcs->wcsproj = WCS_OLD;
+
+	/* if (wcs->wcsproj == WCS_OLD && (
 	    wcs->prjcode != WCS_STG && wcs->prjcode != WCS_AIT &&
 	    wcs->prjcode != WCS_MER && wcs->prjcode != WCS_GLS &&
 	    wcs->prjcode != WCS_ARC && wcs->prjcode != WCS_TAN &&
 	    wcs->prjcode != WCS_TNX && wcs->prjcode != WCS_SIN &&
 	    wcs->prjcode != WCS_PIX && wcs->prjcode != WCS_LIN &&
-	    wcs->prjcode != WCS_CAR && wcs->prjcode != WCS_COE))
-	    wcs->oldwcs = 0;
+	    wcs->prjcode != WCS_CAR && wcs->prjcode != WCS_COE &&
+	    wcs->prjcode != WCS_NCP))
+	    wcs->wcsproj = WCS_NEW; */
 
-	/* Handle NOAO corrected TNX as TAN if oldwcs is set */
-	if (wcs->oldwcs && wcs->prjcode == WCS_TNX) {
+	/* Handle NOAO corrected TNX as uncorrected TAN if oldwcs is set */
+	if (wcs->wcsproj == WCS_OLD && wcs->prjcode == WCS_TNX) {
 	    wcs->ctype[0][6] = 'A';
 	    wcs->ctype[0][7] = 'N';
 	    wcs->prjcode = WCS_TAN;
 	    }
 	}
 
-    /* If not linear or sky coordinates, drop out with error message */
+    /* If not sky coordinates, assume linear */
     else {
-	setwcserr ("WCSTYPE: CTYPE1 not sky coordinates or LINEAR -> no WCS");
-	return (1);
+	wcs->prjcode = WCS_LIN;
+	return (0);
 	}
 
     /* Second coordinate type */
@@ -503,12 +554,14 @@ char	*ctype2;	/* FITS WCS projection for axis 2 */
 	    if (wcs->ctype[1][i] == ' ') wcs->ctype[1][i] = '-';
 	}
 
-    /* If not linear or sky coordinates, drop out with error message */
+    /* If not sky coordinates, assume linear */
     else {
-	setwcserr ("WCSTYPE: CTYPE2 not sky coordinates or LINEAR -> no WCS");
-	return (1);
+	wcs->prjcode = WCS_LIN;
 	}
 
+    /* Set distortion code from CTYPE1 extension */
+    setdistcode (wcs, ctype1);
+
     return (0);
 }
 
@@ -523,9 +576,6 @@ double cdelt1, cdelt2;		/* scale in degrees/pixel, ignored if cd is not NULL */
 double crota;			/* Rotation angle in degrees, ignored if cd is not NULL */
 double *cd;			/* Rotation matrix, used if not NULL */
 {
-    int i, j;
-    double *pci;
-    extern int matinv();
 
     if (nowcs (wcs))
 	return (-1);
@@ -699,10 +749,9 @@ double crota;		/* Rotation counterclockwise in degrees */
 {
     extern int matinv();
     double *pci;
-    double crot, srot, s;
-    int i, j, naxes;
+    double crot, srot;
+    int i, j;
 
-    naxes = wcs->naxes;
     wcs->cdelt[0] = cdelt1;
     if (cdelt2 != 0.0)
 	wcs->cdelt[1] = cdelt2;
@@ -801,7 +850,7 @@ double crota;		/* Rotation counterclockwise in degrees */
 	    }
 	else if (wcs->cdelt[0] > 0 && wcs->cdelt[1] > 0) {
 	    wcs->imflip = 1;
-	    wcs->imrot = wcs->rot;
+	    wcs->imrot = -wcs->rot;
 	    wcs->pa_north = wcs->imrot + 90.0;
 	    if (wcs->pa_north > 180.0) wcs->pa_north = wcs->pa_north - 360.0;
 	    wcs->pa_east = wcs->rot;
@@ -833,13 +882,12 @@ double *pc;		/* Rotation matrix, ignored if NULL */
 {
     extern int matinv();
     double *pci, *pc0i;
-    double crot, srot, s;
     int i, j, naxes;
 
     if (pc == NULL)
 	return;
 
-    naxes = wcs->naxes;
+    naxes = wcs->naxis;
     wcs->cdelt[0] = cdelt1;
     if (cdelt2 != 0.0)
 	wcs->cdelt[1] = cdelt2;
@@ -900,7 +948,7 @@ struct WorldCoor *wcs;	/* World coordinate system structure */
 {
     int i, mem, naxes;
 
-    naxes = wcs->naxes;
+    naxes = wcs->naxis;
     mem = naxes * naxes * sizeof(double);
     if (wcs->lin.piximg == NULL)
 	wcs->lin.piximg = (double*)malloc(mem);
@@ -954,6 +1002,19 @@ struct WorldCoor *wcs;	/* World coordinate system structure */
     int off;
     double cra, cdec, xc, xn, xe, yc, yn, ye;
 
+    /* If image is one-dimensional, leave rotation angle alone */
+    if (wcs->nxpix < 1.5 || wcs->nypix < 1.5) {
+	wcs->imrot = wcs->rot;
+	wcs->pa_north = wcs->rot + 90.0;
+	wcs->pa_east = wcs->rot + 180.0;
+	return;
+	}
+
+
+    /* Do not try anything if image is LINEAR (not Cartesian projection) */
+    if (wcs->syswcs == WCS_LINEAR)
+	return;
+
     wcs->xinc = fabs (wcs->xinc);
     wcs->yinc = fabs (wcs->yinc);
 
@@ -1127,10 +1188,10 @@ struct WorldCoor *wcs;		/* World coordinate system structure */
 	(void) pix2wcs (wcs,xpix,wcs->nypix,&xpos2,&ypos2);
 	if (wcs->syswcs == WCS_LINEAR) {
 	    height = ypos2 - ypos1;
-	    if (width < 100.0)
-	    (void)fprintf (stderr, " height = %.5f %s ",width, wcs->units[1]);
+	    if (height < 100.0)
+	    (void)fprintf (stderr, " height = %.5f %s ",height, wcs->units[1]);
 	    else
-	    (void)fprintf (stderr, " height = %.3f %s ",width, wcs->units[1]);
+	    (void)fprintf (stderr, " height = %.3f %s ",height, wcs->units[1]);
 	    }
 	else {
 	    height = wcsdist (xpos1,ypos1,xpos2,ypos2);
@@ -1211,30 +1272,39 @@ double	*width;		/* Width in degrees (returned) */
 double	*height;	/* Height in degrees (returned) */
 
 {
-    double xpix, ypix, xpos1, xpos2, ypos1, ypos2;
+    double xpix, ypix, xpos1, xpos2, ypos1, ypos2, xcpix, ycpix;
     double xcent, ycent;
 
     /* Find right ascension and declination of coordinates */
     if (iswcs(wcs)) {
-	xpix = 0.5 * wcs->nxpix;
-	ypix = 0.5 * wcs->nypix;
-	(void) pix2wcs (wcs,xpix,ypix,&xcent, &ycent);
+	xcpix = (0.5 * wcs->nxpix) + 0.5;
+	ycpix = (0.5 * wcs->nypix) + 0.5;
+	(void) pix2wcs (wcs,xcpix,ycpix,&xcent, &ycent);
 	*cra = xcent;
 	*cdec = ycent;
 
 	/* Compute image width in degrees */
-	(void) pix2wcs (wcs,1.0,ypix,&xpos1,&ypos1);
-	(void) pix2wcs (wcs,wcs->nxpix,ypix,&xpos2,&ypos2);
+	xpix = 0.500001;
+	(void) pix2wcs (wcs,xpix,ycpix,&xpos1,&ypos1);
+	xpix = wcs->nxpix + 0.499999;
+	(void) pix2wcs (wcs,xpix,ycpix,&xpos2,&ypos2);
 	if (strncmp (wcs->ptype,"LINEAR",6) &&
-	    strncmp (wcs->ptype,"PIXEL",5))
+	    strncmp (wcs->ptype,"PIXEL",5)) {
 	    *width = wcsdist (xpos1,ypos1,xpos2,ypos2);
+	    if (xpos1 < xpos2 && wcs->xinc < 0)
+		*width = 360.0 - *width;
+	    else if (xpos1 > xpos2 && wcs->xinc > 0)
+		*width = 360.0 - *width;
+	    }
 	else
 	    *width = sqrt (((ypos2-ypos1) * (ypos2-ypos1)) +
 		     ((xpos2-xpos1) * (xpos2-xpos1)));
 
 	/* Compute image height in degrees */
-	(void) pix2wcs (wcs,xpix,1.0,&xpos1,&ypos1);
-	(void) pix2wcs (wcs,xpix,wcs->nypix,&xpos2,&ypos2);
+	ypix = 0.5;
+	(void) pix2wcs (wcs,xcpix,ypix,&xpos1,&ypos1);
+	ypix = wcs->nypix + 0.5;
+	(void) pix2wcs (wcs,xcpix,ypix,&xpos2,&ypos2);
 	if (strncmp (wcs->ptype,"LINEAR",6) &&
 	    strncmp (wcs->ptype,"PIXEL",5))
 	    *height = wcsdist (xpos1,ypos1,xpos2,ypos2);
@@ -1254,6 +1324,64 @@ double	*height;	/* Height in degrees (returned) */
 }
 
 
+/* Return minimum and maximum RA and Dec of image in degrees */
+
+void
+wcsrange (wcs, ra1, ra2, dec1, dec2)
+
+struct WorldCoor *wcs;	/* World coordinate system structure */
+double	*ra1;		/* Minimum right ascension of image (deg) (returned) */
+double	*ra2;		/* Maximum right ascension of image (deg) (returned) */
+double	*dec1;		/* Minimum declination of image (deg) (returned) */
+double	*dec2;		/* Maximum declination of image (deg) (returned) */
+
+{
+    double xpos1, xpos2, xpos3, xpos4, ypos1, ypos2, ypos3, ypos4;
+
+    if (iswcs(wcs)) {
+
+	/* Compute image corner coordinates in degrees */
+	(void) pix2wcs (wcs,1.0,1.0,&xpos1,&ypos1);
+	(void) pix2wcs (wcs,1.0,wcs->nypix,&xpos2,&ypos2);
+	(void) pix2wcs (wcs,wcs->nxpix,1.0,&xpos3,&ypos3);
+	(void) pix2wcs (wcs,wcs->nxpix,wcs->nypix,&xpos4,&ypos4);
+
+	/* Find minimum right ascension or longitude */
+	*ra1 = xpos1;
+	if (xpos2 < *ra1) *ra1 = xpos2;
+	if (xpos3 < *ra1) *ra1 = xpos3;
+	if (xpos4 < *ra1) *ra1 = xpos4;
+
+	/* Find maximum right ascension or longitude */
+	*ra2 = xpos1;
+	if (xpos2 > *ra2) *ra2 = xpos2;
+	if (xpos3 > *ra2) *ra2 = xpos3;
+	if (xpos4 > *ra2) *ra2 = xpos4;
+
+	/* Find minimum declination or latitude */
+	*dec1 = ypos1;
+	if (ypos2 < *dec1) *dec1 = ypos2;
+	if (ypos3 < *dec1) *dec1 = ypos3;
+	if (ypos4 < *dec1) *dec1 = ypos4;
+
+	/* Find maximum declination or latitude */
+	*dec2 = ypos1;
+	if (ypos2 > *dec2) *dec2 = ypos2;
+	if (ypos3 > *dec2) *dec2 = ypos3;
+	if (ypos4 > *dec2) *dec2 = ypos4;
+	}
+
+    else {
+	*ra1 = 0.0;
+	*ra2 = 0.0;
+	*dec1 = 0.0;
+	*dec2 = 0.0;
+	}
+
+    return;
+}
+
+
 /* Compute distance in degrees between two sky coordinates */
 
 double
@@ -1297,6 +1425,30 @@ double	x2,y2;	/* (RA,Dec) or (Long,Lat) in degrees */
 }
 
 
+/* Compute distance in degrees between two sky coordinates  away from pole */
+
+double
+wcsdiff (x1,y1,x2,y2)
+
+double	x1,y1;	/* (RA,Dec) or (Long,Lat) in degrees */
+double	x2,y2;	/* (RA,Dec) or (Long,Lat) in degrees */
+
+{
+    double xdiff, ydiff, ycos, diff;
+
+    ycos = cos (degrad ((y2 + y1) / 2.0));
+    xdiff = x2 - x1;
+    if (xdiff > 180.0)
+	xdiff = xdiff - 360.0;
+    if (xdiff < -180.0)
+	xdiff = xdiff + 360.0;
+    xdiff = xdiff / ycos;
+    ydiff = (y2 - y1);
+    diff = sqrt ((xdiff * xdiff) + (ydiff * ydiff));
+    return (diff);
+}
+
+
 /* Initialize catalog search command set by -wcscom */
 
 void
@@ -1333,36 +1485,31 @@ char	*command;		/* command with %s where coordinates will go */
 /* Execute Unix command with world coordinates (from x,y) and/or filename */
 
 void
-wcscom ( wcs, i, filename, xfile, yfile )
+wcscom ( wcs, i, filename, xfile, yfile, wcstring )
 
 struct WorldCoor *wcs;		/* World coordinate system structure */
 int	i;			/* Number of command (0-9) to execute */
 char	*filename;		/* Image file name */
 double	xfile,yfile;		/* Image pixel coordinates for WCS command */
+char	*wcstring;		/* WCS String from pix2wcst() */
 {
-    char wcstring[32];
-    int lstr = 32;
     char command[120];
     char comform[120];
     char xystring[32];
     char *fileform, *posform, *imform;
     int ier;
 
-    if (nowcs (wcs))
+    if (nowcs (wcs)) {
+	(void)fprintf(stderr,"WCSCOM: no WCS\n");
 	return;
+	}
 
     if (wcs->command_format[i] != NULL)
 	strcpy (comform, wcs->command_format[i]);
     else
 	strcpy (comform, "sgsc -ah %s");
 
-    if (nowcs (wcs))
-	(void)fprintf(stderr,"WCSCOM: no WCS\n");
-
-    else if (comform[0] > 0) {
-
-	/* Get WCS coordinates for this image coordinate */
-	(void) pix2wcst (wcs,xfile,yfile,wcstring,lstr);
+    if (comform[0] > 0) {
 
 	/* Create and execute search command */
 	fileform = strsrch (comform,"%f");
@@ -1737,12 +1884,20 @@ int	lstr;		/* Length of world coordinate string (returned) */
 	else if (wcs->degout == 0) {
 	    minlength = 18 + (2 * wcs->ndec);
 	    if (lstr > minlength) {
-		ra2str (rastr, 32, xpos, wcs->ndec);
-		dec2str (decstr, 32, ypos, wcs->ndec-1);
-		if (wcs->tabsys)
+		if (wcs->sysout == WCS_J2000 || wcs->sysout == WCS_B1950) {
+		    ra2str (rastr, 32, xpos, wcs->ndec);
+		    dec2str (decstr, 32, ypos, wcs->ndec-1);
+		    }
+		else {
+		    dec2str (rastr, 32, xpos, wcs->ndec);
+		    dec2str (decstr, 32, ypos, wcs->ndec);
+		    }
+		if (wcs->tabsys) {
 		    (void)sprintf (wcstring,"%s	%s", rastr, decstr);
-		else
+		    }
+		else {
 		    (void)sprintf (wcstring,"%s %s", rastr, decstr);
+		    }
 	        lstr = lstr - minlength;
 		}
 	    else {
@@ -1757,52 +1912,60 @@ int	lstr;		/* Length of world coordinate string (returned) */
 	/* Label galactic coordinates */
 	if (wcs->sysout == WCS_GALACTIC) {
 	    if (lstr > 9 && wcs->printsys)
-		strcat (wcstring," galactic");
+		if (wcs->tabsys)
+		    strcat (wcstring,"	galactic");
+		else
+		    strcat (wcstring," galactic");
 	    }
 
 	/* Label ecliptic coordinates */
 	else if (wcs->sysout == WCS_ECLIPTIC) {
-	    if (lstr > 9 && wcs->printsys)
+	    if (lstr > 9 && wcs->printsys) {
 		if (wcs->tabsys)
 		    strcat (wcstring,"	ecliptic");
 		else
 		    strcat (wcstring," ecliptic");
+		}
 	    }
 
 	/* Label planet coordinates */
 	else if (wcs->sysout == WCS_PLANET) {
-	    if (lstr > 9 && wcs->printsys)
+	    if (lstr > 9 && wcs->printsys) {
 		if (wcs->tabsys)
 		    strcat (wcstring,"	planet");
 		else
 		    strcat (wcstring," planet");
+		}
 	    }
 
 	/* Label alt-az coordinates */
 	else if (wcs->sysout == WCS_ALTAZ) {
-	    if (lstr > 7 && wcs->printsys)
+	    if (lstr > 7 && wcs->printsys) {
 		if (wcs->tabsys)
 		    strcat (wcstring,"	alt-az");
 		else
 		    strcat (wcstring," alt-az");
+		}
 	    }
 
 	/* Label north pole angle coordinates */
 	else if (wcs->sysout == WCS_NPOLE) {
-	    if (lstr > 7 && wcs->printsys)
+	    if (lstr > 7 && wcs->printsys) {
 		if (wcs->tabsys)
 		    strcat (wcstring,"	long-npa");
 		else
 		    strcat (wcstring," long-npa");
+		}
 	    }
 
 	/* Label south pole angle coordinates */
 	else if (wcs->sysout == WCS_SPA) {
-	    if (lstr > 7 && wcs->printsys)
+	    if (lstr > 7 && wcs->printsys) {
 		if (wcs->tabsys)
 		    strcat (wcstring,"	long-spa");
 		else
 		    strcat (wcstring," long-spa");
+		}
 	    }
 
 	/* Label equatorial coordinates */
@@ -1876,14 +2039,14 @@ struct WorldCoor *wcs;		/* World coordinate system structure */
 double	xpix,ypix;	/* x and y image coordinates in pixels */
 double	*xpos,*ypos;	/* RA and Dec in degrees (returned) */
 {
-    double	xp,yp;
+    double	xpi, ypi, xp, yp;
     double	eqin, eqout;
     int wcspos();
     extern int dsspos();
     extern int platepos();
     extern int worldpos();
     extern int tnxpos();
-    extern void fk4prec(),fk5prec(), wcscon();
+    extern void wcscon();
 
     if (nowcs (wcs))
 	return;
@@ -1892,34 +2055,42 @@ double	*xpos,*ypos;	/* RA and Dec in degrees (returned) */
     wcs->zpix = zpix;
     wcs->offscl = 0;
 
+    /* If this WCS is converted from another WCS rather than pixels, convert now */
+    if (wcs->wcs != NULL) {
+	pix2wcs (wcs->wcs, xpix, ypix, &xpi, &ypi);
+	}
+    else {
+	pix2foc (wcs, xpix, ypix, &xpi, &ypi);
+	}
+
     /* Convert image coordinates to sky coordinates */
 
-/* Use Digitized Sky Survey plate fit */
+    /* Use Digitized Sky Survey plate fit */
     if (wcs->prjcode == WCS_DSS) {
-	if (dsspos (xpix, ypix, wcs, &xp, &yp))
+	if (dsspos (xpi, ypi, wcs, &xp, &yp))
 	    wcs->offscl = 1;
 	}
 
     /* Use SAO plate fit */
     else if (wcs->prjcode == WCS_PLT) {
-	if (platepos (xpix, ypix, wcs, &xp, &yp))
+	if (platepos (xpi, ypi, wcs, &xp, &yp))
 	    wcs->offscl = 1;
 	}
 
     /* Use NOAO IRAF corrected plane tangent projection */
     else if (wcs->prjcode == WCS_TNX) {
-	if (tnxpos (xpix, ypix, wcs, &xp, &yp))
+	if (tnxpos (xpi, ypi, wcs, &xp, &yp))
 	    wcs->offscl = 1;
 	}
 
     /* Use Classic AIPS projections */
-    else if (wcs->oldwcs == 1 || wcs->prjcode <= 0) {
-	if (worldpos (xpix, ypix, wcs, &xp, &yp))
+    else if (wcs->wcsproj == WCS_OLD || wcs->prjcode <= 0) {
+	if (worldpos (xpi, ypi, wcs, &xp, &yp))
 	    wcs->offscl = 1;
 	}
 
     /* Use Mark Calabretta's WCSLIB projections */
-    else if (wcspos (xpix, ypix, wcs, &xp, &yp))
+    else if (wcspos (xpi, ypi, wcs, &xp, &yp))
 	wcs->offscl = 1;
 	    	
 
@@ -1974,11 +2145,13 @@ struct WorldCoor *wcs;	/* World coordinate system structure */
 double	xpos,ypos;	/* World coordinates in degrees */
 char	*coorsys;	/* Input world coordinate system:
 			   FK4, FK5, B1950, J2000, GALACTIC, ECLIPTIC
-			   fk4, fk5, b1950, j2000, galactic, ecliptic */
+			   fk4, fk5, b1950, j2000, galactic, ecliptic
+			   * If NULL, use image WCS */
 double	*xpix,*ypix;	/* Image coordinates in pixels */
 int	*offscl;	/* 0 if within bounds, else off scale */
 {
-    double xp,yp;
+    struct WorldCoor *depwcs;	/* Dependent WCS structure */
+    double xp, yp, xpi, ypi;
     double eqin, eqout;
     int sysin;
     int wcspix();
@@ -1986,7 +2159,7 @@ int	*offscl;	/* 0 if within bounds, else off scale */
     extern int platepix();
     extern int worldpix();
     extern int tnxpix();
-    extern void fk4prec(), fk5prec(), wcscon();
+    extern void wcscon();
 
     if (nowcs (wcs))
 	return;
@@ -1998,8 +2171,14 @@ int	*offscl;	/* 0 if within bounds, else off scale */
 	yp = 90.0 - yp;
     else if (wcs->latbase == -90)
 	yp = yp - 90.0;
-    sysin = wcscsys (coorsys);
-    eqin = wcsceq (coorsys);
+    if (coorsys == NULL) {
+	sysin = wcs->syswcs;
+	eqin = wcs->equinox;
+	}
+    else {
+	sysin = wcscsys (coorsys);
+	eqin = wcsceq (coorsys);
+	}
     wcs->zpix = 1.0;
 
     /* Convert coordinates to same system as image */
@@ -2010,39 +2189,47 @@ int	*offscl;	/* 0 if within bounds, else off scale */
 
     /* Use Digitized Sky Survey plate fit */
     if (wcs->prjcode == WCS_DSS) {
-	if (dsspix (xp, yp, wcs, xpix, ypix))
+	if (dsspix (xp, yp, wcs, &xpi, &ypi))
 	    *offscl = 1;
 	}
 
     /* Use SAO polynomial plate fit */
     else if (wcs->prjcode == WCS_PLT) {
-	if (platepix (xp, yp, wcs, xpix, ypix))
+	if (platepix (xp, yp, wcs, &xpi, &ypi))
 	    *offscl = 1;
 	}
 
     /* Use NOAO IRAF corrected plane tangent projection */
     else if (wcs->prjcode == WCS_TNX) {
-	if (tnxpix (xp, yp, wcs, xpix, ypix))
+	if (tnxpix (xp, yp, wcs, &xpi, &ypi))
 	    *offscl = 1;
 	}
 
     /* Use Classic AIPS projections */
-    else if (wcs->oldwcs == 1 || wcs->prjcode <= 0) {
-	if (worldpix (xp, yp, wcs, xpix, ypix))
+    else if (wcs->wcsproj == WCS_OLD || wcs->prjcode <= 0) {
+	if (worldpix (xp, yp, wcs, &xpi, &ypi))
 	    *offscl = 1;
 	}
 
     /* Use Mark Calabretta's WCSLIB projections */
-    else if (wcspix (xp, yp, wcs, xpix, ypix)) {
+    else if (wcspix (xp, yp, wcs, &xpi, &ypi)) {
 	*offscl = 1;
 	}
 
-    /* Set off-scale flag to 2 if off image but within bounds of projection */
-    if (!*offscl) {
-	if (*xpix < 0.5 || *ypix < 0.5)
-	    *offscl = 2;
-	else if (*xpix > wcs->nxpix + 0.5 || *ypix > wcs->nypix + 0.5)
-	    *offscl = 2;
+    /* If this WCS is converted from another WCS rather than pixels, convert now */
+    if (wcs->wcs != NULL) {
+	wcsc2pix (wcs->wcs, xpi, ypi, NULL, xpix, ypix, offscl);
+	}
+    else {
+	foc2pix (wcs, xpi, ypi, xpix, ypix);
+
+	/* Set off-scale flag to 2 if off image but within bounds of projection */
+	if (!*offscl) {
+	    if (*xpix < 0.5 || *ypix < 0.5)
+		*offscl = 2;
+	    else if (*xpix > wcs->nxpix + 0.5 || *ypix > wcs->nypix + 0.5)
+		*offscl = 2;
+	    }
 	}
 
     wcs->offscl = *offscl;
@@ -2050,6 +2237,14 @@ int	*offscl;	/* 0 if within bounds, else off scale */
     wcs->ypos = ypos;
     wcs->xpix = *xpix;
     wcs->ypix = *ypix;
+
+    /* If this WCS is converted to another WCS rather than pixels, convert now */
+    if (wcs->wcsdep != NULL) {
+	xpos = *xpix;
+	ypos = *ypix;
+	depwcs = wcs->wcsdep;
+	wcsc2pix (wcs->wcsdep, xpos, ypos, depwcs->radecin, xpix, ypix, offscl);
+	}
     return;
 }
 
@@ -2108,7 +2303,6 @@ double  *ypix;          /* y pixel number  (dec or lat without rotation) */
 
 {
     int offscl;
-    int i;
     int wcsfwd();
     double wcscrd[4], imgcrd[4], pixcrd[4];
     double phi, theta;
@@ -2121,6 +2315,10 @@ double  *ypix;          /* y pixel number  (dec or lat without rotation) */
 	}
 
     /* Set input for WCSLIB subroutines */
+    wcscrd[0] = 0.0;
+    wcscrd[1] = 0.0;
+    wcscrd[2] = 0.0;
+    wcscrd[3] = 0.0;
     wcscrd[wcs->wcsl.lng] = xpos;
     wcscrd[wcs->wcsl.lat] = ypos;
 
@@ -2205,14 +2403,13 @@ wcserr ()
 
 /* Flag to use AIPS WCS subroutines instead of WCSLIB */
 void
-setdefwcs (oldwcs)
-int oldwcs;
-{ oldwcs0 = oldwcs; return; }
+setdefwcs (wp)
+int wp;
+{ wcsproj0 = wp; return; }
 
 int
 getdefwcs ()
-{ return (oldwcs0); }
-
+{ return (wcsproj0); }
 
 /* Save output default coordinate system */
 static char wcscoor0[16];
@@ -2233,6 +2430,7 @@ static char *wcscom0[10];
 
 void
 savewcscom (i, wcscom)
+int i;
 char *wcscom;
 {
     int lcom;
@@ -2250,7 +2448,7 @@ setwcscom (wcs)
 struct WorldCoor *wcs;  /* WCS parameter structure */
 {
     char envar[16];
-    int lcom, i;
+    int i;
     char *str;
     if (nowcs(wcs))
 	return;
@@ -2264,15 +2462,15 @@ struct WorldCoor *wcs;  /* WCS parameter structure */
 	else if ((str = getenv (envar)) != NULL)
 	    wcscominit (wcs, i, str);
 	else if (i == 1)
-	    wcscominit (wcs, i, "suac -ah %s");
+	    wcscominit (wcs, i, "sua2 -ah %s");	/* F1= Search USNO-A2.0 Catalog */
 	else if (i == 2)
-	    wcscominit (wcs, i, "sgsc -ah %s");
+	    wcscominit (wcs, i, "sgsc -ah %s");	/* F2= Search HST GSC */
 	else if (i == 3)
-	    wcscominit (wcs, i, "stycho -ah %s");
+	    wcscominit (wcs, i, "sty2 -ah %s"); /* F3= Search Tycho-2 Catalog */
 	else if (i == 4)
-	    wcscominit (wcs, i, "sppm -ah %s");
+	    wcscominit (wcs, i, "sppm -ah %s");	/* F4= Search PPM Catalog */
 	else if (i == 5)
-	    wcscominit (wcs, i, "ssao -ah %s");
+	    wcscominit (wcs, i, "ssao -ah %s");	/* F5= Search SAO Catalog */
 	else
 	    wcs->command_format[i] = NULL;
 	}
@@ -2489,4 +2687,42 @@ struct WorldCoor *wcs;  /* WCS parameter structure */
  * Mar 16 1999	Always return (0,0) from wcs2pix if offscale
  * Apr  7 1999	Add code to put file name in error messages
  * Apr  7 1999	Document utility subroutines at end of file
+ * May  6 1999	Fix bug printing height of LINEAR image
+ * Jun 16 1999	Add wcsrange() to return image RA and Dec limits
+ * Jul  8 1999	Always use FK5 and FK4 instead of J2000 and B1950 in RADECSYS
+ * Aug 16 1999	Print dd:mm:ss dd:mm:ss if not J2000 or B1950 output
+ * Aug 20 1999	Add WCS string argument to wcscom(); don't compute it there
+ * Aug 20 1999	Change F3 WCS command default from Tycho to ACT
+ * Oct 15 1999	Free wcs using wcsfree()
+ * Oct 21 1999	Drop declarations of unused functions after lint
+ * Oct 25 1999	Drop out of wcsfree() if wcs is null pointer
+ * Nov 17 1999	Fix bug which caused software to miss NCP projection
+ *
+ * Jan 24 2000	Default to AIPS for NCP, CAR, and COE proj.; if -z use WCSLIB
+ * Feb 24 2000	If coorsys is null in wcsc2pix, wcs->radecin is assumed
+ * May 10 2000	In wcstype(), default to WCS_LIN, not error (after Bill Joye)
+ * Jun 22 2000	In wcsrotset(), leave rotation angle alone in 1-d image
+ * Jul  3 2000	Initialize wcscrd[] to zero in wcspix()
+ *
+ * Feb 20 2001	Add recursion to wcs2pix() and pix2wcs() for dependent WCS's
+ * Mar 20 2001	Add braces to avoid ambiguity in if/else groupings
+ * Mar 22 2001	Free WCS structure in wcsfree even if it is not filled
+ * Sep 12 2001	Fix bug which omitted tab in pix2wcst() galactic coord output
+ *
+ * Mar  7 2002	Fix bug which gave wrong pa's and rotation for reflected RA
+ *		(but correct WCS conversions!)
+ * Mar 28 2002	Add SZP projection
+ * Apr  3 2002	Synchronize projection types with other subroutines
+ * Apr  3 2002	Drop special cases of projections
+ * Apr  9 2002	Implement inversion of multiple WCSs in wcsc2pix()
+ * Apr 25 2002	Use Tycho-2 catalog instead of ACT in setwcscom()
+ * May 13 2002	Free WCSNAME in wcsfree()
+ *
+ * Mar 31 2003	Add distcode to wcstype()
+ * Apr  1 2003	Add calls to foc2pix() in wcs2pix() and pix2foc() in pix2wcs()
+ * May 20 2003	Declare argument i in savewcscom()
+ * Sep 29 2003	Fix bug to compute width and height correctly in wcsfull()
+ * Sep 29 2003	Fix bug to deal with all-sky images orrectly in wcsfull()
+ * Oct  1 2003	Rename wcs->naxes to wcs->naxis to match WCSLIB 3.2
+ * Nov  3 2003	Set distortion code by calling setdistcode() in wcstype()
  */
