@@ -9,7 +9,7 @@ static char SccsId[] = "%W%  %G%";
  * Xlib calls:	XCreateWindow(), XMapWindow()
  * Xlib calls:	XSetStandardProperties(), XSetWMHints()
  * Xlib calls:	XWindowEvent(), XIfEvent(), XGetGeometry()
- * Copyright:	1989 Smithsonian Astrophysical Observatory
+ * Copyright:	1989-2001 Smithsonian Astrophysical Observatory
  *		You may do anything you like with this file except remove
  *		this copyright.  The Smithsonian Astrophysical Observatory
  *		makes no representations about the suitability of this
@@ -17,6 +17,9 @@ static char SccsId[] = "%W%  %G%";
  *		express or implied warranty.
  * Modified:	{0} Michael VanHilst	initial version		30 April 1989
  *		{1} David Muchmore (astro.umontreal.ca) for SGI  23 June 1990
+ *		{2} Sylvain Korzennik (skorzennik@cfa.harvard.edu)
+ *		    for 24B display support                  19 December 2001 
+ *		{3} Doug Mink fix call to XIfEvent           3 September 2003
  *		{n} <who> -- <does what> -- <when>
  */
 
@@ -55,7 +58,7 @@ void create_window ( window, title, argc, argv, set, map )
   window->ID =
     XCreateWindow(window->display, window->parent, window->hints.x,
 		  window->hints.y, window->hints.width, window->hints.height,
-		  window->bdrwidth, CopyFromParent, InputOutput,
+		  window->bdrwidth,  window->depth, InputOutput,
 		  window->visual, window->valuemask, &window->attrs);
 /* Old code (prior to SGI version which reserves its own colormap     *
  *		  CopyFromParent, window->valuemask, &window->attrs); */
@@ -148,7 +151,7 @@ void get_window_dimensions ( window, wait, discard )
 	XWindowEvent(window->display, window->ID, ExposureMask, &event);
       } while( event.xexpose.count != 0 );
     } else {
-      XIfEvent(window->display, &event, check_expose, window);
+      XIfEvent(window->display, &event, check_expose, (char *)window);
     }
   }
   (void)XGetGeometry(window->display, window->ID, &parent,
