@@ -22,6 +22,7 @@ static char SccsId[] = "%W%  %G%";
  * Modified:	{0} Michael VanHilst	initial version	      28 January 1989
  *		{1} MVH			Revised for X11		26 April 1989
  *		{2} Paul Sydney include stdlib.h for malloc()     9 July 1998
+ *              {3} SGK added debug info for 24B                  14 Dec 2001 
  *		{n} <who> -- <does what> -- <when>
  */
 
@@ -212,6 +213,10 @@ static void set_window_basics ( window, parent, border_pixel, background,
   window->image.depth = color.screen_depth;
   window->visual = color.visual;
   window->attrs.colormap = color.colormap;
+#ifdef DEBUG
+  printf("set_window_basics(): depths = %d %d %d\n",
+         color.screen_depth, window->depth, window->image.depth);
+#endif
   if( color.map.private_used )
     window->valuemask |= CWColormap;
   if( color.colormap_mode != VOP_Halftone )
